@@ -1,8 +1,8 @@
 # Vend::Cart - Interchange shopping cart management routines
 #
-# $Id: Cart.pm,v 2.17 2007/08/09 13:40:53 pajamian Exp $
+# $Id: Cart.pm,v 2.23 2008-03-25 17:13:21 jon Exp $
 #
-# Copyright (C) 2002-2007 Interchange Development Group
+# Copyright (C) 2002-2008 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -25,7 +25,7 @@
 
 package Vend::Cart;
 
-$VERSION = substr(q$Revision: 2.17 $, 10);
+$VERSION = substr(q$Revision: 2.23 $, 10);
 
 use strict;
 
@@ -174,6 +174,10 @@ sub DESTROY { }
 
 # If the user has put in "0" for any quantity, delete that item
 # from the order list.
+#
+# Also adjust the cart to take minimum and maximum order quantities
+# into account.
+#
 sub toss_cart {
 	my($s, $cartname) = @_;
 	my $i;
@@ -185,7 +189,11 @@ sub toss_cart {
 	$quantity_raise_event = $raise_event && $quantity_raise_event;
 	my $event_cartname = $cartname || $Vend::CurrentCart;
 	my $old_item;
+	my %quantity_cache;
+
 	DELETE: for (;;) {
+		my %total_quantity = ();
+
 		foreach $i (0 .. $#$s) {
 			my $item = $s->[$i];
 			if ($sub = $Vend::Cfg->{ItemAction}{$s->[$i]{code}}) {
@@ -217,14 +225,14 @@ sub toss_cart {
 						$col = $tab;
 						$tab = $item->{mv_ib} || $Vend::Cfg->{ProductFiles}[0];
 					}
-					$item->{mv_min_quantity} = ::tag_data($tab, $col, $item->{code})
+					$item->{mv_min_quantity} = $quantity_cache{"$tab.$col.$item->{code}"} || ($quantity_cache{"$tab.$col.$item->{code}"} = ::tag_data($tab, $col, $item->{code}))
 											 || '';
 				}
 
 				if(
 					length $item->{mv_min_quantity}
 					and 
-					$item->{quantity} < $item->{mv_min_quantity}
+					$item->{quantity} + $total_quantity{$item->{code}} < $item->{mv_min_quantity}
 					)
 				{
 					$old_item = { %$item } if $quantity_raise_event;
@@ -240,12 +248,20 @@ sub toss_cart {
 			}
 
 			if($Vend::Cfg->{MaxQuantityField}) {
-				my ($tab, $col) = split /:+/, $Vend::Cfg->{MaxQuantityField};
-				if(! length $col) {
-					$col = $tab;
-					$tab = $item->{mv_ib} || $Vend::Cfg->{ProductFiles}[0];
+				$item->{mv_max_quantity} = 0;
+
+				foreach my $fieldspec (split('[,\s]+', $Vend::Cfg->{MaxQuantityField})) {
+					next unless $fieldspec;
+
+					my ($tab, $col) = split /:+/, $fieldspec;
+					if(! length $col) {
+						$col = $tab;
+						$tab = $item->{mv_ib} || $Vend::Cfg->{ProductFiles}[0];
+					}
+					$item->{mv_max_quantity} += $quantity_cache{"$tab.$col.$item->{code}"} || ($quantity_cache{"$tab.$col.$item->{code}"} = ::tag_data($tab, $col, $item->{code}));
 				}
-				$item->{mv_max_quantity} = ::tag_data($tab, $col, $item->{code});
+				$item->{mv_max_quantity} -= $total_quantity{$item->{code}};
+				$item->{mv_max_quantity} = 0 if $item->{mv_max_quantity} < 0;
 
 				if(
 					length $item->{mv_max_quantity}
@@ -256,6 +272,7 @@ sub toss_cart {
 					$old_item = { %$item } if $quantity_raise_event;
 					$item->{quantity} = $item->{mv_max_quantity};
 					$item->{mv_max_over} = 1;
+					delete $item->{mv_min_under};
 					trigger_update(
 							$s,
 							$item,
@@ -265,6 +282,8 @@ sub toss_cart {
 				}
 			}
 
+			$total_quantity{$item->{code}} += $item->{quantity};
+
 			next unless $::Limit->{cart_quantity_per_line}
 				and $item->{quantity} > $::Limit->{cart_quantity_per_line};
 			
