@@ -1,8 +1,8 @@
 # Vend::Order - Interchange order routing routines
 #
-# $Id: Order.pm,v 2.94 2007/08/09 13:40:53 pajamian Exp $
+# $Id: Order.pm,v 2.100.2.2 2008-10-24 10:19:06 pajamian Exp $
 #
-# Copyright (C) 2002-2007 Interchange Development Group
+# Copyright (C) 2002-2008 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -29,7 +29,7 @@
 package Vend::Order;
 require Exporter;
 
-$VERSION = substr(q$Revision: 2.94 $, 10);
+$VERSION = substr(q$Revision: 2.100.2.2 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -1118,7 +1118,7 @@ EOF
 # NF = Newfoundland is deprecated and will be removed at some point;
 # use NL instead
 $state_template{CA} = <<EOF;
-| AB BC MB NB NF NL NS NT ON PE QC SK YT YK |
+| AB BC MB NB NF NL NS NT NU ON PE QC SK YT YK |
 EOF
 
 $zip_error{US} = "'%s' not a US zip code";
@@ -1780,11 +1780,13 @@ sub route_order {
 			$pagefile = $route->{'report'} || $main->{'report'};
 			$page = readfile($pagefile);
 		}
-		die errmsg(
-			"No order report %s or %s found.",
-			$route->{'report'},
-			$main->{'report'},
-			) unless defined $page;
+		unless (defined $page) {
+			my $msg = errmsg("No order report %s or %s found.",
+							 $route->{'report'},
+							 $main->{'report'});
+			::logError("$msg\n");
+			die("$msg\n");
+		}
 
 		my $use_mime;
 		undef $::Instance->{MIME};
@@ -1825,8 +1827,10 @@ sub route_order {
 			$reply   = $::Values->{$reply} if $reply =~ /^\w+$/;
 			$to		 = $route->{email};
 			my $ary = [$to, $subject, $page, $reply, $use_mime];
-			if($route->{from}) {
-				push @$ary, "From: $route->{from}";
+			for (qw/from bcc cc/) {
+				if ($route->{$_}) {
+					push @$ary, ucfirst($_) . ": $route->{$_}";
+				}
 			}
 			push @out, $ary;
 		}
@@ -2321,8 +2325,9 @@ sub add_items {
 		for (@sku) {
 			$_ = $::Variable->{MV_VARIANT_JOINER} || '0' if ! length($_);
 		}
-		$skus[0]   = $items[0];
 		$items[0] = join '-', @sku;
+		my $sku_field = $Vend::Cfg->{Options_repository}{Matrix}->{sku} || 'sku';
+		$skus[0] = Vend::Data::product_field($sku_field, $items[0]);
 	}
 
 	if ($Vend::Cfg->{UseModifier}) {
