@@ -1,6 +1,6 @@
 # Vend::Parse - Parse Interchange tags
 # 
-# $Id: Parse.pm,v 2.40 2007/08/09 13:40:53 pajamian Exp $
+# $Id: Parse.pm,v 2.44 2007-12-19 12:33:44 pajamian Exp $
 #
 # Copyright (C) 2002-2007 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
@@ -36,7 +36,7 @@ require Exporter;
 
 @ISA = qw(Exporter Vend::Parser);
 
-$VERSION = substr(q$Revision: 2.40 $, 10);
+$VERSION = substr(q$Revision: 2.44 $, 10);
 
 @EXPORT = ();
 @EXPORT_OK = qw(find_matching_end);
@@ -248,28 +248,14 @@ my %attrAlias = (
 						},
 );
 
-my %attrDefault;
+my %attrDefault = ();
 
 my %Alias = (
+	getlocale	=> 'setlocale get=1',
+	process_search	=> 'area href=search',
+);
 
-				qw(
-						url				urldecode
-						urld			urldecode
-						href			area
-						warning			warnings
-						shipping_description	shipping_desc
-						process_target	process
-						process_order	process
-				),
-					getlocale		=> 'setlocale get=1',
-					process_search		=> 'area href=search',
-			);
-
-my %Interpolate = (
-
-				qw(
-				)
-			);
+my %Interpolate = ();
 
 my %NoReparse = ( qw/
 					restrict		1
@@ -661,6 +647,7 @@ sub start {
 		if(defined $Alias{$tag}) {
 			$aliasname = $tag;
 			my $alias = $Alias{$tag};
+			$alias =~ tr/-/_/;
 			$tag =~ s/_/[-_]/g;
 #::logDebug("origtext: $origtext tag=$tag alias=$alias");
 			$origtext =~ s/$tag/$alias/i
@@ -778,7 +765,7 @@ EOF
 			$$buf = '';
 			$Initialized->{_buf} = '';
 			
-            my $body = qq{Redirecting to <A href="%s">%s</a>.};
+            my $body = qq{Redirecting to <a href="%s">%s</a>.};
             $body = errmsg($body, $attr->{href}, $attr->{href});
 #::logDebug("bouncing...body=$body");
 			$::Pragma->{download} = 1;
@@ -821,18 +808,24 @@ EOF
 			$Vend::CurrentTag = $tagsave;
 			$tmpbuf = $p->{ABORT} ? '' : ${$p->{OUT}};
 		}
-		if($attr->{reparse} ) {
+		if ($attr->{'hide'}) {
+			$routine->(@args,$tmpbuf);
+		}
+		elsif($attr->{reparse} ) {
 			$$buf = ($routine->(@args,$tmpbuf)) . $$buf;
 		}
 		else {
-			${$self->{OUT}} .= &{$routine}(@args,$tmpbuf);
+			${$self->{OUT}} .= $routine->(@args,$tmpbuf);
 		}
 	}
-	elsif(! $attr->{interpolate}) {
-		${$self->{OUT}} .= &$routine( @args );
+	elsif ($attr->{'hide'}) {
+		$routine->(@args);
+	}
+	elsif($attr->{interpolate}) {
+		$$buf = $routine->(@args) . $$buf;
 	}
 	else {
-		$$buf = &$routine( @args ) . $$buf;
+		${$self->{OUT}} .= $routine->(@args);
 	}
 
 	$self->{SEND} = $attr->{'send'} || undef;
