@@ -1,8 +1,8 @@
 # Vend::Table::GDBM - Access an Interchange table stored in a GDBM file
 #
-# $Id: GDBM.pm,v 2.16 2007/08/09 13:40:56 pajamian Exp $
+# $Id: GDBM.pm,v 2.19 2008-04-19 14:38:14 jon Exp $
 #
-# Copyright (C) 2002-2007 Interchange Development Group
+# Copyright (C) 2002-2008 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -28,9 +28,10 @@ use strict;
 use vars qw($VERSION @ISA);
 use GDBM_File;
 use Vend::Table::Common;
+use Encode qw(encode decode);
 
 @ISA = qw(Vend::Table::Common);
-$VERSION = substr(q$Revision: 2.16 $, 10);
+$VERSION = substr(q$Revision: 2.19 $, 10);
 
 sub new {
 	my ($class, $obj) = @_;
@@ -57,6 +58,8 @@ sub create {
 	my $dbm = tie(%$tie, 'GDBM_File', $filename, $flags, $File_permission_mode)
 		or die ::errmsg("%s %s: %s\n", ::errmsg("create"), $filename, $!);
 
+	apply_utf8_filters($dbm) if $config->{GDBM_ENABLE_UTF8};
+
 	$tie->{'c'} = join("\t", @$columns);
 
 	my $s = [
@@ -86,9 +89,8 @@ sub open_table {
 		$flags |= GDBM_NOLOCK if $config->{IC_LOCKING};
 		if(! defined $config->{AutoNumberCounter}) {
 			eval {
-				my $dot = $config->{HIDE_AUTO_FILES} ? '.' : '';
 				$config->{AutoNumberCounter} = new Vend::CounterFile
-									"$config->{DIR}/$dot$config->{name}.autonumber",
+									$config->{AUTO_NUMBER_FILE},
 									$config->{AUTO_NUMBER} || '00001',
 									$config->{AUTO_NUMBER_DATE};
 			};
@@ -114,6 +116,8 @@ sub open_table {
 	die ::errmsg("%s could not tie to '%s': %s", 'GDBM', $filename, $!)
 		unless $dbm;
 
+	apply_utf8_filters($dbm) if $config->{GDBM_ENABLE_UTF8};
+
 	my $columns = [split(/\t/, $tie->{'c'})];
 
 	$config->{VERBATIM_FIELDS} = 1 unless defined $config->{VERBATIM_FIELDS};
@@ -132,6 +136,22 @@ sub open_table {
 	bless $s, $class;
 }
 
+sub apply_utf8_filters {
+	my ($handle) = shift;
+
+#::logDebug("applying UTF-8 filters to GDBM handle");
+
+	my $out_filter = sub { $_ = encode('utf-8', $_) };
+	my $in_filter  = sub { $_ = decode('utf-8', $_) };
+
+	$handle->filter_store_key($out_filter);
+	$handle->filter_store_value($out_filter);
+	$handle->filter_fetch_key($in_filter);
+	$handle->filter_fetch_value($in_filter);
+
+	return $handle;
+}
+
 # Unfortunate hack need for Safe searches
 *column_index	= \&Vend::Table::Common::column_index;
 *column_exists	= \&Vend::Table::Common::column_exists;
